#ifndef CCHERWIG_HH 
#define CCHERWIG_HH

#include "AGILe/Generator.hh"
#include "ThePEG/Repository/EventGenerator.h"


namespace AGILe {

  /// Abstract base class for the C++-based Herwig generator series
  /// @author Lars Sonnenschein
  /// @author Andy Buckley
  /// @date   2007-09-28


  class CCHerwig : public Generator {
  public:
    /// Standard constructor.
    CCHerwig();
    
    /// Destructor.
    ~CCHerwig() { }

    /// Set up initial state from supplied params.
    Status setGenSpecificInitialState(PdgCode p1 = PROTON, double e1 = 7000.0, 
                           PdgCode p2 = PROTON, double e2 = 7000.0);
    
    /// Run the generator for one event.
    Status makeEvent(HepMC::GenEvent& evt);

    /// Set the random number generator seed.
    virtual Status setSeed(const int value);
  
    /// Make sure that generic setParam is visible.
    using Generator::setParam;
 
    /// Pass a string parameter to the generator.
    Status setParam(const string& name, const string& value);
    
    /// Tidy up after ourselves
    Status finalize();
    
  protected:
    /// Set up default params etc.
    Status initialize();

    /// Do nothing, since common block not involved.
    Status fillEvent(HepMC::GenEvent& evt) { return SUCCESS; }
   
  private:
    ThePEG::EGPtr eg;

  };
  
}

#endif
  
