// -*- C++ -*-
#include "AGILe/AlpGen/AlpGenFHerwig.hh"
#include "AGILe/AlpGen/AlpGenWrapper.hh"
#include "AGILe/FHerwig/FHerwigWrapper65.hh"
#include "AGILe/Utils.hh"
#include "HepMC/HEPEVT_Wrapper.h"


//-----------------------------------------------------------------------------
// Implementation file for class : AlpGenFHerwig
// This is the file-reading interface wrapper only. It contains an instance of
// FHerwig which does most of the work. This class is only to provide access to 
// the parameter 
// settings.
//
// Authors: Jon Butterworth
//          Andy Buckley
//-----------------------------------------------------------------------------


namespace AGILe {

  // Standard constructor
  AlpGenFHerwig::AlpGenFHerwig() {
    // Set flag to read from LHA event file
    FC_HWPROC.IPROC = -100;

    // Set the ALPGEN filename to "zhjj" by default
    FC_AHIO.FILENAME[0] = 'z';
    FC_AHIO.FILENAME[1] = 'h';
    FC_AHIO.FILENAME[2] = 'j';
    FC_AHIO.FILENAME[3] = 'j';
  }


  // Set string parameters
  Status AlpGenFHerwig::setParam(const string& name, const string& value) {
    Generator::setParam(name, value);

    // Strings
    if (name == "ALPGENFILE") {
      cout << "Setting ALPGENFILE = " << value << endl;
      size_t nch = (value.size() < 100) ? value.size() : 99;
      for (size_t i = 0; i < 100; ++i) {
        if (i < nch) {
          FC_AHIO.FILENAME[i] = value.data()[i];
        } else {
          FC_AHIO.FILENAME[i] = ' ';
        }
      }
    }

    // Integers
    else if (name == "IEXC") {
      cout << "Setting ALPGEN IEXC = " << value << endl;
      FC_AHOPTS.IEXC = asInt(value);
    }

    // Doubles
    else if (name == "ETACLMAX") { 
      cout << "Setting ETACLMAX = " << value << endl;
      FC_AHOPTS.ETACLMAX = asDouble(value); 
    } 

    // Fallback
    else if (FHerwig::setParam(name, value) != SUCCESS) {
      cerr << "AlpGenFHerwig doesn't have a string-valued parameter called " << name << endl;
      return FAILURE;
    }
    return SUCCESS;
  }

  //call event generation with veto routine for MLM matching
  Status AlpGenFHerwig::makeEvent(HepMC::GenEvent& evt) {
    Generator::makeEvent(evt);
    FC_HWUINE();  // Initialize event
    FC_HWEPRO();  // Generate hard subprocess
    //call the matching routine
    int imatch = 0;
    FC_UPVETO(&imatch);
    if (imatch != 0){
      //cerr << "Event Killed by UPVETO" << endl;
      FC_HWUFNE();
      return SUCCESS;
    }
    FC_HWBGEN();  // Generate parton cascade
    FC_HWDHOB();  // Do heavy quark decays
    FC_HWCFOR();  // Do cluster formation
    FC_HWCDEC();  // Do cluster decays
    FC_HWDHAD();  // Do unstable particle decays
    FC_HWDHVY();  // Do heavy flavor decays
    FC_HWMEVT();  // Add soft underlying event
    FC_HWUFNE();  // Finish event
    fillEvent(evt);
    return SUCCESS;
  }

}


// Class factory
extern "C" {
  AGILe::Generator* create() { return new AGILe::AlpGenFHerwig(); }
  void destroy(AGILe::Generator* gen) { delete gen; }
}
