// -*- C++ -*-
#include "AGILe/Charybdis/CharybdisFPythia.hh"
#include "AGILe/Charybdis/CharybdisWrapper.hh"
#include "AGILe/FPythia/FPythia.hh"
#include "AGILe/FPythia/FPythiaWrapper62.hh"
#include "AGILe/Utils.hh"

//-----------------------------------------------------------------------------
// Implementation file for class : CharybdisFPythia
// This is the file-reading interface only. It needs to be used in conjunction 
// with FPythia. This class is only to provide access to the parameter 
// settings.
// Authors: Doug Gingrich
//          Andy Buckley
//-----------------------------------------------------------------------------


namespace AGILe {

  // Constructor
  CharybdisFPythia::CharybdisFPythia() {
    FC_HEPRUP.IDBMUP[0] = 2212;
    FC_HEPRUP.IDBMUP[1] = 2212;
    FC_HEPRUP.EBMUP[0] = 7000.0;
    FC_HEPRUP.EBMUP[1] = 7000.0;
    FC_HEPRUP.PDFGUP[0] = -1;
    FC_HEPRUP.PDFGUP[1] = -1;
    FC_HEPRUP.PDFSUP[0] = -1;
    FC_HEPRUP.PDFSUP[1] = -1;
    FC_PYPARS.mstp[142] = 1;
  }


  // Set string parameters
  Status CharybdisFPythia::setParam(const string& name, const string& value) {
    Generator::setParam(name, value);

    // Set double-valued params
    if (name == "MPLNCK") {
      cout << "Setting ... (MPLNCK) = " << value << endl;
      FC_BHPARM.MPLNCK = asDouble(value);
    } else if (name == "MINMSS") {
      cout << "Setting ... (MINMSS) = " << value << endl;
      FC_BHPARM.MINMSS = asDouble(value);
    } else if (name == "MAXMSS") {
      cout << "Setting ... (MAXMSS) = " << value << endl;
      FC_BHPARM.MAXMSS = asDouble(value);
    } else if (name == "THWMAX") {
      cout << "Setting ... (THWMAX) = " << value << endl;
      FC_BH1003.THWMAX = asDouble(value);
    } else if (name == "RMMINM") {
      cout << "Setting ... (RMMINM) = " << value << endl;
      FC_BH1003.RMMINM = asDouble(value);

    // Set int-valued params
    } else if (name == "PDFGUP") {
      cout << "Setting Charybdis PDFGUP = " << value << endl;
      FC_HEPRUP.PDFGUP[0] = asInt(value);
      FC_HEPRUP.PDFGUP[1] = asInt(value);
    } else if (name == "PDFSUP") {
      cout << "Setting Charybdis = " << value << endl;
      FC_HEPRUP.PDFSUP[0] = asInt(value);
      FC_HEPRUP.PDFSUP[1] = asInt(value);
    } else if (name == "MSSDEF") {
      cout << "Setting ... (MSSDEF) = " << value << endl;
      FC_BHPARM.MSSDEF = asInt(value);
    } else if (name == "NBODY") {
      cout << "Setting ... (NBODY) = " << value << endl;
      FC_BHPARM.NBODY = asInt(value);
    } else if (name == "IBHPRN") {
      cout << "Setting ... (IBHPRN) = " << value << endl;
      FC_BHPARM.IBHPRN = asInt(value);
    } else if (name == "MSSDEC") {
      cout << "Setting ... (MSSDEC) = " << value << endl;
      FC_BHPARM.MSSDEC = asInt(value);
    } else if (name == "TOTDIM") {
      cout << "Setting ... (TOTDIM) = " << value << endl;
      FC_BLACKH.TOTDIM = asInt(value);

    // Set bool-valued params
    } else if (name == "TIMVAR") {
      cout << "Setting ... (TIMVAR) = " << value << endl;
      FC_BHPARM.TIMVAR = asInt(value);
    } else if (name == "GTSCA") {
      cout << "Setting ... (GTSCA) = " << value << endl;
      FC_BHPARM.GTSCA = asInt(value);
    } else if (name == "GRYBDY") {
      cout << "Setting ... (GRYBDY) = " << value << endl;
      FC_BHPARM.GRYBDY = asInt(value);
    } else if (name == "KINCUT") {
      cout << "Setting ... (KINCUT) = " << value << endl;
      FC_BHPARM.KINCUT = asInt(value);
    } else if (name == "YRCSEC") {
      cout << "Setting ... (YRCSEC) = " << value << endl;
      FC_BH1003.YRCSEC = asInt(value);
    } else if (name == "RMBOIL") {
      cout << "Setting ... (RMBOIL) = " << value << endl;
      FC_BH1003.RMBOIL = asInt(value);
    } else if (FPythia::setParam(name, value) != SUCCESS) {
      cerr << "CharybdisFPythia doesn't have a parameter called " << name << endl;
      return FAILURE;
    }
    return SUCCESS;
  }

}


// Class factory
extern "C" {
  AGILe::Generator* create() { return new AGILe::CharybdisFPythia(); }
  void destroy(AGILe::Generator* gen) { delete gen; }
}
